@@ -12,7 +12,7 @@ module SassSpec::CLI
       skip: false,
       verbose: false,
       filter: "",
-      limit: -1,
+      implementation: 'sass'
     }
 
     OptionParser.new do |opts|
@@ -48,18 +48,21 @@ Make sure the command you provide prints to stdout.
 
       opts.on("-c", "--command COMMAND", "Sets a specific binary to run") do |v|
         options[:engine_adapter] = ExecutableEngineAdapter.new(v)
+        options[:engine_adapter].args = options[:cmd_args]
       end
 
       opts.on("--dart PATH", "Run Dart Sass, whose repo should be at the given path.") do |path|
         options[:engine_adapter] = DartEngineAdapter.new(path)
-        options[:engine_adapter].args = options[:dart_args]
+        options[:engine_adapter].args = options[:cmd_args]
       end
 
-      opts.on("--dart-args ARGS", "Pass ARGS to Dart Sass.") do |args|
+      opts.on("--cmd-args ARGS", "Pass ARGS to command or Dart Sass.") do |args|
         if (adapter = options[:engine_adapter]) && adapter.is_a?(DartEngineAdapter)
           adapter.args = args
+        elsif (adapter = options[:engine_adapter]) && adapter.is_a?(ExecutableEngineAdapter)
+          adapter.args = args
         else
-          options[:dart_args] = args
+          options[:cmd_args] = args
         end
       end
 
@@ -84,6 +87,7 @@ Make sure the command you provide prints to stdout.
       end
 
       opts.on("--limit NUMBER", "Limit the number of tests run to this positive integer.") do |limit|
+        raise "--limit must receive a positive integer" if limit.to_i < 0
         options[:limit] = limit.to_i
       end
 
@@ -95,7 +99,7 @@ Make sure the command you provide prints to stdout.
         options[:silent] = true
       end
 
-      opts.on("--check-annotations", "Check if any test annotations are unecessary.") do
+      opts.on("--check-annotations", "Check if any test annotations are unnecessary.") do
         options[:check_annotations] = true
       end
 
